"""Model representing the Housie board"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from typing import List, Optional
from random import choice

from housie.constants import SelectedPool, RemainingPool, Number, NUMBER_POOL
from housie.utils import clear_screen


class Board:
	"""Represents the Housie board state and logic to run the game and display the state"""

	def __init__(self, already_selected: Optional[SelectedPool] = None) -> None:
		"""Initialize the state of the board.
		Either a blank new board, or some pre-selected numbers to represent an already mid-way game"""
		# Variables for holding game state
		self.remaining: RemainingPool = []
		self.selected: SelectedPool = []
		if already_selected:
			self.init_custom_game(already_selected)
		else:
			self.init_new_game()

	def pick_next(self) -> Optional[Number]:
		"""Picks a number at random from the remaining pool of numbers.
		The chosen number is removed from the remaining_pool and added to the selected pool"""
		if not self.remaining:
			return None
		number = choice(self.remaining)
		return self.pick_manual(number)

	def pick_many(self, count: int) -> List[Number]:
		"""Pick multiple numbers from the remaining pool of numbers
		The chosen numbers are removed from the remaining pool and added to the selected pool"""
		return list(filter(None, [self.pick_next() for i in range(count)]))

	def pick_manual(self, number: Number) -> Number:
		"""Manually pick a number from the number pool.
		The chosen number is removed from the remaining pool and added to the selected pool"""
		if number in self.remaining:
			self.remaining.remove(number)
			self.selected.append(number)
		return number

	def init_new_game(self) -> None:
		"""Initialize a new game"""
		self.selected = []
		self.remaining = NUMBER_POOL.copy()

	def init_custom_game(self, already_selected: list) -> None:
		"""Initialize a new game with a set of numbers already called out"""
		self.selected = already_selected
		self.remaining = list(set(NUMBER_POOL) - set(already_selected))

	def last_5_selected(self) -> List[Number]:
		"""Returns a list of the last 5 numbers called out"""
		return self.selected[-5:]

	def display_board(self) -> str:
		"""Display the Housie Board of selected numbers visually in the console"""
		board = ''
		board += '------------------ Housie Board -------------------\n'
		row_size = 10
		num_rows = len(NUMBER_POOL) // row_size
		for row_num in range(num_rows):
			start = row_num * row_size
			end = (row_num * row_size) + row_size
			for num in NUMBER_POOL[start: end]:
				if num in self.selected:
					board += '|{: 3} '.format(num)
				else:
					board += '|' + ' '*4
			board += '|'
			board += '\n---------------------------------------------------\n'

		last_5_numbers_str = ", ".join(map(str, reversed(self.last_5_selected())))
		board += 'Last 5 Numbers called: {}\n'.format(last_5_numbers_str)
		board += 'Left in the bag: {}\n'.format(len(self.remaining))
		return board


def demo_board() -> None:
	"""Test function to print out how the board would look with a few numbers filled"""
	housie_board = Board()
	housie_board.pick_many(40)
	clear_screen()
	print(housie_board.display_board())



"""Reusable utilities across the project"""
from typing import Any

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


import os
import json
from functools import wraps


def clear_screen() -> None:
	"""Clears the console"""
	# for windows
	if os.name == 'nt':
		os.system('cls')

	# for mac and linux(here, os.name is 'posix')
	else:
		os.system('clear')


def strike_through(text: str) -> str:
	"""Returns a strike-through version of the input text"""
	result = ''
	for c in text:
		result = result + c + '\u0336'
	return result


def create_data_dir_if_not_exists() -> None:
	"""Creates a data directory if it does not exists. The data directory holds all game relevant data files"""
	from housie.constants import DATA_DIR
	if not os.path.exists(DATA_DIR):
		os.makedirs(DATA_DIR)


def save_json(data: Any, filename: str) -> None:
	"""Saves the input data to a file"""
	create_data_dir_if_not_exists()
	with open(filename, 'w') as file:
		json.dump(data, file)


def load_json(filename: str) -> Any:
	"""Loads and returns the json data from a file as json. If the file is missing, returns None"""
	try:
		with open(filename) as file:
			return json.load(file)
	except FileNotFoundError:
		return None


def dynamic_doc(func):
	"""Decorator to insert the actual values of variables into the docstrings of certain methods.

	This is so that we can insert certain values from our constants into our docstrings to make them more accurate
	"""
	@wraps(func)
	def wrapper(*args, **kwargs):
		"""No added behavior in the wrapper apart from the formatted docstring"""
		return func(*args, **kwargs)
	# Format the original docstring with the constant values
	from housie import constants
	wrapper.__doc__ = func.__doc__.format(**constants.__dict__)
	return wrapper



"""Convenience file to help start the game when the repo is cloned from git rather than installed via pip

This was required as we needed to run the script from the same level as the housie/ package in order for the imports
to work correctly.
"""
from housie.game import display_main_menu

display_main_menu()



"""Main Logic to drive the Game-Play

Classic Housie Board game

Also known as Bingo
https://en.wikipedia.org/wiki/Bingo_(British_version)
"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from typing import Dict, List

from housie.utils import load_json, save_json, clear_screen, dynamic_doc
from housie.constants import INSTRUCTIONS, Number, FOLLOW_GAME_TICKETS_NOT_FOUND_MSG, FOLLOWED_TICKETS_FILE, \
	FOLLOWED_BOARD_FILE, GENERATED_TICKETS_FILE
from housie.models import Board, Ticket, load_tickets
from housie.display_util import display_followed_game
from housie.generate_ticket import generate_ticket


def print_options() -> str:
	"""Prints the options to allow the user to drive the program flow"""
	option = 'default'
	while option not in ['N', 'T', 'F', 'Q']:
		clear_screen()
		print(INSTRUCTIONS)
		if option != 'default':
			print("Sorry that is not a valid input. Please try again")
		option = input("\nSelect an option to get started: ").upper()
	return option


def host_new_game() -> None:
	"""Host a Housie Game.
	Starts with a blank board. Randomly picks numbers and updates the board when you ask it to
	"""
	options = "Press 'Enter' to pick the next number\nPress 'Q' followed by 'Enter' to quit\n"
	board = Board()
	clear_screen()
	print(board.display_board())
	user_choice = ''
	while user_choice not in ['Q', 'q']:
		user_choice = input(options)
		board.pick_next()
		clear_screen()
		print(board.display_board())


@dynamic_doc
def follow_game() -> None:
	"""Allows you to play along/follow a game being hosted by someone else.

	Reads your tickets from '{FOLLOWED_TICKETS_FILE}' and displays them.
	Refer to the '{FOLLOWED_TICKETS_EXAMPLE_FILE}' file as a reference file.

	Reads the board from {FOLLOWED_BOARD_FILE} on startup. Displays the board.
	Allows you to enter the numbers being called out by the host, and updates your tickets and the board.
	The state of the board is persisted even if you exit the program.
	To start a new game, delete the {FOLLOWED_BOARD_FILE} file.

	For this mode, certain files are read from/saved to a folder named '{DATA_DIR}' which
	should be in the current directory from where you are running the game.

	NOTE: This mode doesn't let you generate numbers randomly on the board. This is because the whole purpose of this
	mode is to allow to follow a game being hosted by someone else.

	If you want to host a game and play with friends, use the generate tickets mode to distribute tickets to your
	friends, and then use the host a game mode to play.
	"""
	already_selected_numbers = load_json(FOLLOWED_BOARD_FILE)
	board = Board(already_selected_numbers)
	ticket_data: Dict[str, List[Ticket]] = load_tickets(FOLLOWED_TICKETS_FILE)
	if not ticket_data:
		print(FOLLOW_GAME_TICKETS_NOT_FOUND_MSG)
		return None
	mark_tickets_full_board(board, ticket_data)
	while True:
		clear_screen()
		display_followed_game(board, ticket_data)
		user_choice = input("Press 'Q' to quit. Enter next number: ")
		if user_choice == 'Q' or user_choice == 'q':
			break
		elif user_choice.isnumeric():
			number = int(user_choice)
			board.pick_manual(number)
			save_json(board.selected, FOLLOWED_BOARD_FILE)
			mark_tickets(number, ticket_data)


def mark_tickets_full_board(board: Board, ticket_data: Dict[str, List[Ticket]]) -> None:
	"""Updates the tickets with all numbers from the housie board"""
	for name, tickets in ticket_data.items():
		for ticket in tickets:
			ticket.mark_numbers(board.selected)


def mark_tickets(number: Number, ticket_data: Dict[str, List[Ticket]]) -> None:
	"""Updates the tickets with all numbers from the housie board"""
	for name, tickets in ticket_data.items():
		for ticket in tickets:
			ticket.mark_number(number)


@dynamic_doc
def generate_tickets() -> None:
	"""Allows you to generate housie tickets for use in a game. Enter the names of the players and numbers of tickets
	per player

	Saves the generated tickets to a file '{GENERATED_TICKETS_FILE}'
	"""
	clear_screen()
	names = input("Enter the names of users playing the game. Separate each name with a space: ")
	number = input("Enter the number of tickets to be generated per user: ")
	ticket_data: Dict[str, List[Ticket]] = {}
	names = map(str.strip, names.split())
	for name in names:
		tickets = [generate_ticket() for i in range(int(number))]
		print(name)
		for ticket in tickets:
			print(ticket.display_ticket())
		# Convert from ticket object in a list of rows representing the ticket so we can save and load it from json
		ticket_data[name] = [ticket.rows for ticket in tickets]
	save_json(ticket_data, GENERATED_TICKETS_FILE)
	print(f"The generated tickets can also be found in the '{GENERATED_TICKETS_FILE}' file")


@dynamic_doc
def display_main_menu() -> None:
	"""The starting menu presented to the user

	Supported Options are as follows :

	'N' - Host a New Game
		Displays the board, randomly picks numbers

	'T' - Generate Tickets
		Generate tickets for Players to use to play the game. Also stores the tickets to the '{GENERATED_TICKETS_FILE}'

	'F' - Follow a Game
		This mode allows you to follow a game being hosted by someone else, i.e. someone else is calling out the
		numbers as you sit and mark your tickets. This mode automates the marking of your tickets and shows a nice
		visual display of the board and your tickets.

		Before using this mode, enter your ticket(s) into the '{FOLLOWED_TICKETS_FILE}' file. Then start this mode
		and keep entering the numbers being called out. The status of your ticket(s) are automatically updated on
		screen.

	'Q' - Quit

	"""
	option = print_options()
	if option == 'N':
		host_new_game()
	elif option == 'F':
		follow_game()
	elif option == 'Q':
		print("Thank you for playing! Bye")
	elif option == 'T':
		generate_tickets()
	else:
		print("Sorry this feature is not available yet!")


if __name__ == '__main__':
	display_main_menu()
	# demo_board()



"""Complex logic for displaying a followed game conveniently in terminal"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from itertools import zip_longest
from typing import Dict, List

from housie.models import Board, Ticket


def display_followed_game(board: Board, ticket_data: Dict[str, List[Ticket]]) -> None:
	"""Use this to select how you want the game displayed. The complex display looks better, but may not work in all
	sizes of terminals. Use the simple display as a fall back"""
	# Side-by-Side display with minimalistic ticket design
	# _complex_display_followed_game(housie, ticket_data, 'minimalistic_display', 24)

	# Side-by-Side display with traditional/structural ticket design
	_complex_display_followed_game(board, ticket_data, 'structural_display', 36)

	# Simplistic vertical scrolling display
	# _simple_display_followed_game(housie, ticket_data)


def _simple_display_followed_game(board: Board, ticket_data: Dict[str, List[Ticket]]) -> None:
	"""Simple Display method for use in following a game

	Displays the board followed by the tickets
	"""
	print(board.display_board())
	for name, tickets in ticket_data.items():
		print(name)
		for ticket in tickets:
			print(ticket.display_ticket())


class DisplayElement:
	"""A wrapper object to group together multiple lines of text as a single element"""
	def __init__(self, content: str) -> None:
		self.row_wise_data = content.split('\n')
		self.content = content


def _complex_display_followed_game(
		board: Board, ticket_data: Dict[str, List[Ticket]], display_method: str, line_len: int) -> None:
	"""Side-by-Side Display method for use in following a game.
	Tries to fit the tickets besides the board instead of below.

	Not the best implementation though, as it involves a lot of string manipulations.
	Also doesn't consider size of the terminal for now. But it works!

	Allows to display up to 8 tickets in a nice clean manner. After that, the view could get a little messy
	"""
	board_element = DisplayElement(board.display_board())
	ticket_elements = []
	for name, tickets in ticket_data.items():
		ticket_str = ''
		for ticket in tickets:
			ticket_str += getattr(ticket, display_method)() + '\n'
		ticket_str = name + (line_len-len(name))*'-' + '\n' + ticket_str
		ticket_elements.append(DisplayElement(ticket_str))

	if len(ticket_elements) > 1:
		combined_element = fold(ticket_elements)
		further_combined = fold([board_element, combined_element])
		print(further_combined.content)
	else:
		combined_element = fold([board_element, ticket_elements[0]])
		print(combined_element.content)


def fold(display_elements: List[DisplayElement]) -> DisplayElement:
	"""Split the incoming list of elements into two side-by-side columns"""
	middle = len(display_elements) // 2
	column1 = display_elements[:middle]
	column2 = display_elements[middle:]

	# Flatten the List of display elements into a list of rows. (each display element contains rows)
	column1_rows = []
	for element in column1:
		for row in element.row_wise_data:
			column1_rows.append(row)

	column2_rows = []
	for element in column2:
		for row in element.row_wise_data:
			column2_rows.append(row)

	combined_str = ''
	space = ' '
	fill_value = len(column1_rows[0]) * ' ' if len(column2_rows) > len(column1_rows) else len(column2_rows[0]) * ' '
	for col1, col2 in zip_longest(column1_rows, column2_rows, fillvalue=fill_value):
		combined_str += col1 + 5 * space + col2 + '\n'
	return DisplayElement(combined_str)



"""Simple Housie Ticket class to store and display a ticket"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from typing import List, Dict, Optional, Set

from housie.constants import Row, COLUMN_RANGES, Number, NUMBER_POOL
from housie.utils import load_json, strike_through, clear_screen


class Ticket:
	"""Simple structure to store and display the ticket"""
	def __init__(self, rows: List[Row]):
		# self.rows will hold only the non-None elements
		self.rows: List[Row] = []
		# Storing all the numbers in a flat list
		self.numbers: List[int] = []
		for row in rows:
			new_row = []
			for number in row:
				if number:
					self.numbers.append(number)
					new_row.append(number)
			self.rows.append(new_row)
		# All the numbers that are selected
		self.selected: Set[Number] = set()

	def mark_number(self, number: int) -> None:
		"""Updates the ticket marking the matching number as selected"""
		if number in self.numbers:
			self.selected.add(number)

	def mark_numbers(self, numbers: List[int]) -> None:
		"""Updates the ticket marking the matching numbers as selected"""
		for number in numbers:
			self.mark_number(number)

	def __repr__(self) -> str:
		return "Ticket(numbers={})".format(self.rows)

	def display_ticket(self) -> str:
		"""Display the ticket"""
		return self.structural_display()

	def minimalistic_display(self) -> str:
		"""Displays the ticket in a minimalistic format"""
		ticket_representation = ''
		for row in self.rows:
			row_elems = []
			for number in row:
				if number:
					if number in self.selected:
						row_elems.append(strike_through('{: 3} '.format(number)))
					else:
						row_elems.append('{: 3} '.format(number))
			ticket_representation += ','.join(row_elems) + '\n'
		return ticket_representation

	def structural_display(self) -> str:
		"""Displays the ticket in a 3 x 9 grid as is seen on traditional tickets"""
		ticket_representation = ''
		for row in self.get_structural_representation():
			for num in row:
				if num:
					if num in self.selected:
						ticket_representation += strike_through('{: 3} '.format(num))
					else:
						ticket_representation += '{: 3} '.format(num)
				else:
					ticket_representation += '  - '
			ticket_representation += '\n'
		return ticket_representation

	def get_structural_representation(self) -> List[Row]:
		"""Returns a representation of the ticket in a 3 x 9 grid with blank squares represented by None

		This representation can be used for a graphical display.
		"""
		representation = []
		for row in self.rows:
			new_row = []
			row_iter = iter(row)
			num = next(row_iter)
			if num:
				for start, end in COLUMN_RANGES:
					if num and start <= num <= end:
						new_row.append(num)
						try:
							num = next(row_iter)
						except StopIteration:
							# Add 'None' to the remaining elements of the row
							new_row.extend([None] * (9 - len(new_row)))
							break
					else:
						new_row.append(None)
				representation.append(new_row)
		return representation


def load_tickets(file_name: str) -> Optional[Dict[str, List[Ticket]]]:
	"""Reads the input file and tries to parse the data into a dict of name : tickets"""
	ticket_data = load_json(file_name)
	if not ticket_data:
		return None
	for name, tickets in ticket_data.items():
		tickets = list(map(Ticket, tickets))
		ticket_data[name] = tickets
	return ticket_data


def demo_ticket() -> None:
	"""Method to demonstrate how tickets would look like with some numbers marked"""
	from housie.generate_ticket import generate_ticket

	# Generate some sample tickets
	names = ['Thor', 'Ironman', 'Spiderman']
	ticket_data = {}
	for name in names:
		ticket = generate_ticket()
		ticket_data[name] = [ticket]

	# Mark some random numbers so we can see the output
	from random import sample
	random_numbers = sample(NUMBER_POOL, 40)
	for name, tickets in ticket_data.items():
		for ticket in tickets:
			for number in random_numbers:
				ticket.mark_number(number)

	# Display the tickets
	clear_screen()
	for name, tickets in ticket_data.items():
		print(name)
		for ticket in tickets:
			print(ticket.display_ticket())



"""Housie Ticket generation logic"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from collections import defaultdict
from random import choice, randint
from typing import List, Set, Dict, DefaultDict

from housie.constants import Number, Row, ColumnRange, COLUMN_RANGES
from housie.models import Ticket


def generate_ticket() -> Ticket:
	"""Generates a Housie Ticket containing 15 randomly selected numbers based on the following rules

	* A Housie ticket has 15 numbers.
	* These 15 numbers are placed in a 3 rows x 9 cols grid (27 possible spaces).
	* There has to be exactly 5 numbers in each row.
	* There has to be at least one number in each column.
	* The numbers in each column must belong to that column's allowed range.
		Column 1: 1-9
		Column 2: 10-19
		Column 3: 20-29
		.      .      .
		.      .      .
		.      .      .
		Column 8: 70-79
		Column 9: 80-90

	Based on the rules above we have to select the numbers.
	"""
	# The list of ranges for each column. Make a copy as we mutate it during our selection process
	column_ranges: List[ColumnRange] = COLUMN_RANGES.copy()

	# The numbers selected for the ticket
	ticket_numbers: Set[Number] = set()

	# Saving the numbers column-wise to ensure we select at max 3 numbers for each column and also
	# to help in distributing them into the 3 rows later
	column_wise_numbers: DefaultDict[ColumnRange, List[Number]] = defaultdict(list)

	# Select one number from each column range. This will give us 9 numbers
	for column_range in column_ranges:
		selected_number = select_unique_number_from_range(
			column_range=column_range, already_selected=column_wise_numbers[column_range])
		ticket_numbers.add(selected_number)
		column_wise_numbers[column_range].append(selected_number)

	# Select the remaining 6 numbers at random from the columns.
	while len(ticket_numbers) < 15:
		selected_range = choice(column_ranges)
		selected_number = select_unique_number_from_range(
			column_range=selected_range, already_selected=column_wise_numbers[selected_range])

		ticket_numbers.add(selected_number)
		column_wise_numbers[selected_range].append(selected_number)

		# If we have selected 3 numbers for a specific column, remove that column from the further selection process
		if len(column_wise_numbers[selected_range]) == 3:
			column_ranges.remove(selected_range)

	# Convert to a regular dict now as we no longer need the defaultdict properties
	# Now we assign the 15 selected numbers into the 3 rows.
	rows = assign_to_rows(dict(column_wise_numbers))
	return Ticket(rows)


def select_unique_number_from_range(column_range: ColumnRange, already_selected: List[Number]) -> Number:
	"""Selects and returns a number randomly from a ColumnRange. If the number has already been selected before, then
	re-select a new number"""
	selected_number: Number = randint(column_range.start, column_range.end)
	# If the selected_number is already_selected, select a new number until we get a unique number
	while selected_number in already_selected:
		selected_number = randint(column_range.start, column_range.end)
	return selected_number


def assign_to_rows(column_wise_numbers: Dict[ColumnRange, List[Number]]) -> List[Row]:
	"""Distribute the ticket numbers into the 3 rows such that each row has exactly 5 numbers

	Start with the columns that have 3 numbers, followed by the columns that have two numbers and
	then columns that have 1 number.
	Assign the numbers in a round-robin fashion.
	"""
	ticket: List[Row] = [[], [], []]
	row_num = 0
	for col_len in [3, 2, 1]:
		columns = list(filter(lambda x: len(x) == col_len, column_wise_numbers.values()))
		for column in columns:
			row_num = insert_from_columns_to_rows(column, ticket, row_num)
	for row in ticket:
		row.sort()
	return ticket


def insert_from_columns_to_rows(column: List[Number], ticket: List[Row], row_num: int) -> int:
	"""Insert numbers from the columns into the rows in a round-robin fashion.
	Returns the row_num so it can be persisted between calls"""
	numbers = sorted(column)

	# If we have to assign numbers to row 3 and row 1 of the same column, we should always assign the smaller
	# number to the higher row, i.e. row 1. To do this, we have the following logic:
	# Figure out which rows we will be assigning the column numbers to. Assign in round robin logic starting from the
	# smallest row num possible.
	row_nums_to_assign_to = []
	for number in numbers:
		row_nums_to_assign_to.append(row_num)
		row_num = (row_num + 1) % 3

	# This is the last row position that we are assigning to in the round-robin assignment. Return this for the
	# assignment of the numbers from the next column
	last_row_num = row_num
	row_nums_to_assign_to.sort()
	for number, row_num in zip(numbers, row_nums_to_assign_to):
		ticket[row_num].append(number)
	return last_row_num


def demo_ticket_generation() -> None:
	"""Demonstrates ticket generation by generating a ticket and printing it visually"""
	sample_ticket = generate_ticket()
	print("Concise representation")
	print(sample_ticket.minimalistic_display())

	print("Structural representation")
	print(sample_ticket.structural_display())



"""Constants file for holding constant values declared throughout the app"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from typing import List, NamedTuple

# Type Aliases
Number = int

# Board related Types
# List of all valid numbers for Housie
NUMBER_POOL: List[Number] = list(range(1, 91))
RemainingPool = List[Number]
SelectedPool = List[Number]

# Ticket Related Types
Row = List[Number]
ColumnRange = NamedTuple('ColumnRange', [('start', Number), ('end', Number)])

# User Menu Selection Choice Type
Choice = str

# List of Valid ranges for each column of the ticket
COLUMN_RANGES = \
		[ColumnRange(1, 9)] + [ColumnRange(start, start + 9) for start in range(10, 71, 10)] + [ColumnRange(80, 90)]

# Constants related to data files
DATA_DIR = 'data'
FOLLOWED_TICKETS_FILE = DATA_DIR + '/followed_tickets.json'
FOLLOWED_BOARD_FILE = DATA_DIR + '/followed_board.json'
GENERATED_TICKETS_FILE = DATA_DIR + '/generated_tickets.json'
FOLLOWED_TICKETS_EXAMPLE_FILE = \
	'https://raw.githubusercontent.com/aaronalphonso/housie/master/src/data/followed_tickets.example.json'


# Main menu instructional message
INSTRUCTIONS = f"""Welcome to Housie!

Press 'N' to Host a New Game 
	Displays board, randomly picks numbers

Press 'T' to Generate Tickets 
	Generate tickets for Players to use to play the game

Press 'F' to Follow a Game 
	This mode allows you to follow a game being hosted by someone else, i.e. someone else is calling out the
	numbers as you sit and mark your tickets. This mode automates the marking of your tickets and shows a nice
	visual display of the board and your tickets. You can also quit and continue the game from where you left off 
	later. (Game state is persisted)
	
	Before using this mode, enter your ticket(s) into '{FOLLOWED_TICKETS_FILE}' file in the current directory. 
	Then start this mode and keep entering the numbers being called out. The status of your ticket(s) are 
	automatically updated on screen. 
	
	(This mode saves files into the '{DATA_DIR}' folder in order to store the 
	state of the housie board. If you want to start a new game, delete the file named '{FOLLOWED_BOARD_FILE}')

Press 'Q' to Quit
"""


FOLLOW_GAME_TICKETS_NOT_FOUND_MSG = f"""
No tickets found in file '{FOLLOWED_TICKETS_FILE}'!

If you want to follow a game, make sure to first add your tickets into this file. 
If the file doesn't exist, create it. Make sure to create the file inside a folder named '{DATA_DIR}' 
in the current directory in which you are running the game. You can use the file found at 
'{FOLLOWED_TICKETS_EXAMPLE_FILE}' as a reference file.
"""



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="housie",
    version="0.1.2",
    author="Aaron Alphonso",
    author_email="alphonsoaaron1993@gmail.com",
    description="All the core logic for playing/simulating the popular game 'Housie' "
                "(also known as 'Bingo' or 'Tambola')",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aaronalphonso/housie",
    packages=['housie', 'housie.models'],
    package_dir={'': 'src'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    keywords='python housie core tambola bingo ticket generator board game',
    project_urls={
        'Source': "https://github.com/aaronalphonso/housie"
    },
)



"""Core models and logic required to simulate a game of Housie

Board - For holding the state of the Housie board and the numbers selected so far.
Ticket - For representing a single Housie ticket and the numbers on it as well as the numbers marked
generate_ticket - A function that generates tickets for use in the game
"""

__author__ = 'Aaron Alphonso'
__email__ = 'alphonsoaaron1993@gmail.com'


from .models import Board, Ticket, demo_board, demo_ticket, load_tickets
from .generate_ticket import generate_ticket, demo_ticket_generation
from .game import display_main_menu

